/*
 * $QNXLicenseC:
 * Copyright 2010, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"
#include <arm/am335x.h>
#include "am335x_pinmux.h"

/* Pinmux for BeagleBoard */

static void init_uart0_pin_mux(void)
{
    out32(conf_uart0_rxd        , (MODE(0) | PULLUP_EN | RXACTIVE | PULLUDEN));    /* UART0_RXD  */  /* (E15) uart0_rxd.uart0_rxd */
    out32(conf_uart0_txd        , (MODE(0) | PULLUDEN ));                          /* UART0_TXD  */  /* (E16) uart0_txd.uart0_txd */
    out32(conf_uart0_ctsn       , (MODE(0) | RXACTIVE | PULLUDEN));                /* UART0_CTSN */  /* (E18) uart0_ctsn.uart0_ctsn */
    out32(conf_uart0_rtsn       , (MODE(0) | PULLUP_EN | PULLUDEN));               /* UART0_RTSN */  /* (E17) uart0_rtsn.uart0_rtsn */
}

static void init_i2c0_pin_mux(void)
{
    out32(conf_i2c0_sda         , (MODE(0) | RXACTIVE | PULLUDEN | SLEWCTRL));     /* I2C_DATA  */  /* (C17) I2C0_SDA.I2C0_SDA */
    out32(conf_i2c0_scl         , (MODE(0) | RXACTIVE | PULLUDEN | SLEWCTRL));     /* I2C_SCLK  */  /* (C16) I2C0_SCL.I2C0_SCL */
}

static void init_mmc0_pin_mux(void)
{
    out32(conf_mmc0_dat3        , (MODE(0) | RXACTIVE | PULLUP_EN)); /* MMC0_DAT3  */  /* (F17) mmc0_dat3.mmc0_dat3 */
    out32(conf_mmc0_dat2        , (MODE(0) | RXACTIVE | PULLUP_EN)); /* MMC0_DAT2  */  /* (F18) mmc0_dat2.mmc0_dat2 */
    out32(conf_mmc0_dat1        , (MODE(0) | RXACTIVE | PULLUP_EN)); /* MMC0_DAT1  */  /* (G15) mmc0_dat1.mmc0_dat1 */
    out32(conf_mmc0_dat0        , (MODE(0) | RXACTIVE | PULLUP_EN)); /* MMC0_DAT0  */  /* (G16) mmc0_dat0.mmc0_dat0 */
    out32(conf_mmc0_clk         , (MODE(0) | RXACTIVE | PULLUP_EN)); /* MMC0_CLK   */  /* (G17) mmc0_clk.mmc0_clk */
    out32(conf_mmc0_cmd         , (MODE(0) | RXACTIVE | PULLUP_EN)); /* MMC0_CMD   */  /* (G18) mmc0_cmd.mmc0_cmd */
 //   out32(conf_spi0_cs1         , (MODE(5) | RXACTIVE | PULLUP_EN)); /* MMC0_CD    */  
	out32(conf_gpmc_a10, MODE(7) |RXACTIVE | PULLUP_EN ); // PIN_INPUT_PULLUP /* (T16) gpmc_a10.gpio1[26] */ // SD card detect

}

static void init_leds_pin_mux(void)
{
    out32(conf_mii1_col       , MODE(7));        /* (H16) gmii1_col.gpio3[0] */ // GPIO3_0
    out32(conf_gpmc_wait0     , MODE(7));        /* (T17) gpmc_wait0.gpio0[30] */ // GPIO0_30
	out32(conf_gpmc_wpn       , MODE(7));        /* (U17) gpmc_wpn.gpio0[31] */  // GPIO0_31
}

// also sets up mdio pins
// This is for RGMII, so 3 less pins than gmii (removed crs,col,rxerr)- TBD rename function!
static void init_gmii1_pin_mux(void)
{
	
    out32(conf_mii1_txclk       , MODE(2) | RXACTIVE);                             /* GMII1_TXCLK  */ /* (K18) gmii1_txclk.rgmii1_tclk */
    out32(conf_mii1_txd0        , MODE(2));                                        /* GMII1_TXD0   */ /* (K17) gmii1_txd0.rgmii1_td0 */
    out32(conf_mii1_txd1        , MODE(2));                                        /* GMII1_TXD1   */ /* (K16) gmii1_txd1.rgmii1_td1 */
    out32(conf_mii1_txd2        , MODE(2));                                        /* GMII1_TXD2   */ /* (K15) gmii1_txd2.rgmii1_td2 */
    out32(conf_mii1_txd3        , MODE(2));                                        /* GMII1_TXD3   */ /* (J18) gmii1_txd3.rgmii1_td3 */
    out32(conf_mii1_txen        , MODE(2));                                        /* GMII1_TXEN   */ /* (J16) gmii1_txen.rgmii1_tctl */ 
    out32(conf_mii1_rxclk       , (MODE(2) | RXACTIVE));                             /* GMII1_RXCLK  */  /* (L18) gmii1_rxclk.rgmii1_rclk */
    out32(conf_mii1_rxd0        , (MODE(2) | RXACTIVE));                             /* GMII1_RXD0   */  /* (M16) gmii1_rxd0.rgmii1_rd0 */
    out32(conf_mii1_rxd1        , (MODE(2) | RXACTIVE));                             /* GMII1_RXD1   */  /* (L15) gmii1_rxd1.rgmii1_rd1 */
    out32(conf_mii1_rxd2        , (MODE(2) | RXACTIVE));                             /* GMII1_RXD2   */  /* (L16) gmii1_rxd2.rgmii1_rd2 */
    out32(conf_mii1_rxd3        , (MODE(2) | RXACTIVE));                             /* GMII1_RXD3   */  /* (L17) gmii1_rxd3.rgmii1_rd3 */
    out32(conf_mii1_rxdv        , (MODE(2) | RXACTIVE));                             /* GMII1_RXDV   */  /* (J17) gmii1_rxdv.rgmii1_rctl */
    out32(conf_mdio_data        , (MODE(0) | RXACTIVE | PULLUP_EN));                 /* MDIO_DATA    */  /* (M17) mdio_data.mdio_data */
    out32(conf_mdio_clk         , (MODE(0)));                                       /* MDIO_CLK     */  /* (M18) mdio_clk.mdio_clk */
}

static void init_glue_pin_mux(void)
{
    out32(conf_nnmi, MODE(0));              /* (B18) nNMI.nNMI */ // NB there is a pullup on the board 

// V17 eeprom
		out32(conf_gpmc_a11, (MODE(7)| RXACTIVE | PULLUP_EN) );   /* (V17) gpmc_a11.gpio1[27] */  // EEPROM Hold
		out32(conf_gpmc_be1n,(MODE(7)| RXACTIVE | PULLUP_EN) );   /* (U18) gpmc_be1n.gpio1[28] */ // WP for EEPROM

// Other
		out32(conf_mcasp0_aclkr, MODE(7));    /* (B12) mcasp0_aclkr.gpio3[18] */  // RELAY - output
    out32(conf_gpmc_advn_ale,MODE(7));    /* (R7)  gpmc_advn_ale.gpio2[2]    */  // TP22 

    out32(conf_mcasp0_ahclkr,(MODE(7) | RXACTIVE ) );  /*  (C12) mcasp0_ahclkr.gpio3[17] */  // this needs to be input, so doesn't conflict with mod to use H18 as SPI1_CS0

			
}

static void init_fpga_pin_mux(void)
{
	// intrs/clk
	  out32(conf_xdma_event_intr0, MODE(3));  /* (A15) xdma_event_intr0.clkout1 */ // clkout to FPGA
	  out32(conf_xdma_event_intr1, MODE(0));  /* (D14) xdma_event_intr1.xdma_event_intr1 */ // intr from FPGA NB there is a pullup on the board 

// GPMC
#if 0 // GPMC not working yet for FPGA programming, using GPIO mode 

// GPMC to control the pins
	  out32(conf_gpmc_ad7, MODE(0)); /* PIN_INPUT  (T9) gpmc_ad7.gpmc_ad7 */ // Mux'd address/data[7]
		out32(conf_gpmc_ad6, MODE(0)); /* PIN_INPUT  (R9) gpmc_ad6.gpmc_ad6 */ //  ...
		out32(conf_gpmc_ad5, MODE(0)); /* PIN_INPUT  (V8) gpmc_ad5.gpmc_ad5 */
		out32(conf_gpmc_ad4, MODE(0)); /* PIN_INPUT  (U8) gpmc_ad4.gpmc_ad4 */
		out32(conf_gpmc_ad3, MODE(0)); /* PIN_INPUT  (T8) gpmc_ad3.gpmc_ad3 */
		out32(conf_gpmc_ad2, MODE(0)); /* PIN_INPUT  (R8) gpmc_ad2.gpmc_ad2 */
		out32(conf_gpmc_ad1, MODE(0)); /* PIN_INPUT  (V7) gpmc_ad1.gpmc_ad1 */
		out32(conf_gpmc_ad0, MODE(0)); /* PIN_INPUT  (U7) gpmc_ad0.gpmc_ad0 */ // Mux'd address/data[0]

		out32(conf_gpmc_csn1,MODE(1)); /* PIN_OUTPUT (U9) gpmc_csn1.gpmc_clk */ // DCLK
    out32(conf_gpmc_csn0,MODE(0)); /* PIN_OUTPUT (V6) gpmc_csn0.gpio1[29] */ // must assign a chip select in GPMC mode, even if we don't use it!

#else
// Control pins as GPIOs
    out32(conf_gpmc_ad7, MODE(7)); /* (T9) gpmc_ad7.gpio1[7] */ // DATA7
		out32(conf_gpmc_ad6, MODE(7)); /* (R9) gpmc_ad6.gpio1[6] */ // ...
		out32(conf_gpmc_ad5, MODE(7)); /* (V8) gpmc_ad5.gpio1[5] */
		out32(conf_gpmc_ad4, MODE(7)); /* (U8) gpmc_ad4.gpio1[4] */
		out32(conf_gpmc_ad3, MODE(7)); /* (T8) gpmc_ad3.gpio1[3] */
		out32(conf_gpmc_ad2, MODE(7)); /* (R8) gpmc_ad2.gpio1[2] */
		out32(conf_gpmc_ad1, MODE(7)); /* (V7) gpmc_ad1.gpio1[1] */ // ..
		out32(conf_gpmc_ad0, MODE(7)); /* (U7) gpmc_ad0.gpio1[0] */ // DATA0

		out32(conf_gpmc_csn1,MODE(7) | RXACTIVE | PULLUDEN);  // PIN_INPUT_PULLDOWN /* (U9) gpmc_csn1.gpio1[30] */ // DCLK
    out32(conf_gpmc_csn0,MODE(7));     /* PIN_OUTPUT (V6) gpmc_csn0.gpio1[29] */ // TP23// GPIO1_29

#endif


		out32(conf_gpmc_csn2,    (MODE(7)| RXACTIVE | PULLUP_EN)); /* (V9) gpmc_csn2.gpio1[31] */  // nCONFIG - output - GPIO1_31 
		out32(conf_gpmc_csn3,    (MODE(7)| RXACTIVE));             /* (T13) gpmc_csn3.gpio2[0] */  // nSTATUS - input - GPIO2_0
		out32(conf_gpmc_wen,     (MODE(7)| RXACTIVE));             /* (U6)  gpmc_wen.gpio2[4]  */  // CONF_DONE GPIO2_4 - input
		
}

// NB: PULLUP_EN /* Pull UP Selection */
 //    PULLUDEN   /* Pull up enabled */

static void init_gpio_pin_mux(void)
{
// all GPIOs as inputs initially

		out32(conf_gpmc_a0 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (R13) gpmc_a0.gpio1[16] */
		out32(conf_gpmc_a1 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (V14) gpmc_a1.gpio1[17] */
		out32(conf_gpmc_a2 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (U14) gpmc_a2.gpio1[18] */
		out32(conf_gpmc_a3 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (T14) gpmc_a3.gpio1[19] */
		out32(conf_gpmc_a4 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (R14) gpmc_a4.gpio1[20] */
		out32(conf_gpmc_a5 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (V15) gpmc_a5.gpio1[21] */
		out32(conf_gpmc_a6 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (U15) gpmc_a6.gpio1[22] */
		out32(conf_gpmc_a7 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (T15) gpmc_a7.gpio1[23] */
		out32(conf_gpmc_a8 , (MODE(7)| RXACTIVE | PULLUP_EN | PULLUDEN));//PIN_INPUT /* (V16) gpmc_a8.gpio1[24] */
		out32(conf_gpmc_a9 , (MODE(7)| RXACTIVE | PULLUDEN)); //PIN_INPUT /* (U16) gpmc_a9.gpio1[25] */
		
}

// These peripherals are always on - not covered under "cape control"
void init_pinmux()
{
    init_i2c0_pin_mux();
    init_leds_pin_mux();
    init_gmii1_pin_mux(); 
    init_mmc0_pin_mux();
    init_uart0_pin_mux();
    init_glue_pin_mux();
    init_fpga_pin_mux();
    init_gpio_pin_mux();
}

/*
 **********************
 **Pinmuxes for Capes**
 **********************
 */

 /*
  * Note that pinmuxes for capes depend on what cape is used. Different capes need
  * different pinmuxes depending on what devices a cape implements (uart, lcd, etc).
  *
  * The functions below are provided as-is only. They demonstrate as an example how
  * a pinmux for a specific device on a cape could be configured. The functions provided
  * are not a complete set, and you may have to implement your own pinmux for your cape.
  * When no cape is present, a default profile of 0 is assumed.
  */

  
/*
 ****************************
 **          UART          **
 ****************************
 */

/* 
 * UART1 - Note: this multiplex choice cannot be used in conjunction with can1 or mmc1.
 * Untested - example configuration for cape uart1
 */
static void init_uart1_pin_mux(void)
{                                                                                /*  [expansion header(P8 or P9)  expansion header pin ]*/
                                                                                    
    out32(conf_uart1_rxd        , (MODE(0) | PULLUP_EN | RXACTIVE));             /* [P9 26] UART1_RXD  */  /* (D16) uart1_rxd.uart1_rxd */
    out32(conf_uart1_txd        , (MODE(0) | PULLUDEN));                         /* [P9 24] UART1_TXD  */  /* (D15) uart1_txd.uart1_txd */
}

/*
 * UART2 - Note: this multiplex choice cannot be used in conjunction with spi1.
 * Untested - example configuration for cape uart2
 */
static void init_uart2_pin_mux(void)
{                                                                        
   out32(conf_spi0_sclk        , (MODE(1) | PULLUP_EN | RXACTIVE));              /* [P9 22] UART2_RXD  */
   out32(conf_spi0_d0          , (MODE(1) | PULLUDEN));                          /* [P9 21] UART2_TXD  */
}

/* UART4 */
static void init_uart4_pin_mux(void)
{                                                                        
    out32(conf_gpmc_wait0       , (MODE(6) | PULLUP_EN | RXACTIVE));             /* [P9 11] UART4_RXD  */
    out32(conf_gpmc_wpn         , (MODE(6) | PULLUDEN));                         /* [P9 13] UART4_TXD  */
}


/*
 ****************************
 **          I2C           **
 ****************************
 */

/*
 * I2C1 - Note: this multiplex choice cannot be used in conjunction with spi0.
 * Untested - example configuration for cape i2c1
 */

static void init_i2c1_pin_mux(void)
{                                                                               
   out32(conf_mii1_rxerr       , MODE(3) | RXACTIVE | PULLUDEN | SLEWCTRL);    /* GMII1_RXERR    (J15) gmii1_rxer.I2C1_SCL */
   out32(conf_mii1_crs         , MODE(3) | RXACTIVE | PULLUDEN | SLEWCTRL);    /* GMIIuart01_CRS (H17) gmii1_crs.I2C1_SDA */ // GMII1_CRS/RMII1_CRS_DV/SPI1_D0/I2C1_SDA/MCASP1_ACLKX/UART5_CTSN/UART2_RXD/GPIO3_1

}

/* I2C2 */
static void init_i2c2_pin_mux(void)
{                                                                                
    out32(conf_uart1_ctsn       , MODE(3) | RXACTIVE | PULLUDEN | SLEWCTRL);    /* [P9 20] I2C2_DATA  */  /* (D18) uart1_ctsn.I2C2_SDA */
    out32(conf_uart1_rtsn       , MODE(3) | RXACTIVE | PULLUDEN | SLEWCTRL);    /* [P9 19] I2C2_SCLK  */  /* (D17) uart1_rtsn.I2C2_SCL */
}

/*
 ****************************
 **          SPI           **
 ****************************
 */

/*
 * SPI0 - Note: this multiplex choice cannot be used in conjunction with i2c1 and uart2.
 * Untested - example configuration for cape spi0
 */

static void init_spi0_pin_mux(void)
{                                                                                
    out32(conf_spi0_sclk        , MODE(0) | PULLUDEN | RXACTIVE);                /* [P9 22] SPI0_SCLK  */ /* (A17) spi0_sclk.spi0_sclk */
    out32(conf_spi0_d0          , MODE(0) | PULLUDEN | PULLUP_EN | RXACTIVE);    /* [P9 21] SPI0_D0    */ /* (B17) spi0_d0.spi0_d0 */
    out32(conf_spi0_d1          , MODE(0) | PULLUDEN | RXACTIVE);                /* [P9 18] SPI0_D1    */ /* (B16) spi0_d1.spi0_d1 */
// TBC guess CS input mode for potential used in slave mode.    
    out32(conf_spi0_cs0         , MODE(0) | PULLUDEN | PULLUP_EN | RXACTIVE);    /* [P9 17] SPI0_CS0   */ /* (A16) spi0_cs0.spi0_cs0 */
    out32(conf_spi0_cs1         , MODE(0) | PULLUDEN | PULLUP_EN | RXACTIVE);  // PIN_OUTPUT /* (C15) spi0_cs1.spi0_cs1 */

}

/*
 * SPI1 - Note: this multiplex choice cannot be used in conjunction with HDMI on BBB
 */
static void init_spi1_pin_mux(void)
{                                                                                
    out32(conf_mcasp0_aclkx     , MODE(3) | PULLUDEN | RXACTIVE);                /* [P9 31] SPI1_SCLK  */  /* (A13) mcasp0_aclkx.spi1_sclk */
    out32(conf_mcasp0_fsx       , MODE(3) | PULLUDEN | PULLUP_EN | RXACTIVE);    /* [P9 29] SPI1_D0    */  /* (B13) mcasp0_fsx.spi1_d0 */
    out32(conf_mcasp0_axr0      , MODE(3) | PULLUDEN | PULLUP_EN | RXACTIVE);    /* [P9 30] SPI1_D1    */  /* (D12) mcasp0_axr0.spi1_d1 */

#if 0 // When 1: Use CS0 for device 0. 
// TBC guess CS input mode for potential used in slave mode.
    out32(conf_mcasp0_ahclkr    , MODE(3) );    /* [P9 28] SPI1_CS0   */  /* (C12) mcasp0_ahclkr.spi1_cs0 */
    out32(conf_ecap0_in_pwm0_out, MODE(7) );    /* [P9 28] SPI1_CS1 - use GPIO0[7] for EEPROM CS  */ /* (C18) eCAP0_in_PWM0_out.gpio0[7] */

#else // for EEPROM on SPI1/CS device1 
    // NB H18 is not connected on LCR board rev1 - needs wire mod, and set C12 (GPIO3[17]) as input
    out32(conf_rmii1_refclk         , MODE(2));    /* [P9 28] SPI1_CS0   */ /* (H18) rmii1_refclk.spi1_cs0 */ // FPGA SPI1_CS0 
    out32(conf_ecap0_in_pwm0_out    , MODE(2));    /* [P9 28] SPI1_CS1   */ /* (C18) eCAP0_in_PWM0_out.spi1_cs1 */ // EEPROM SPI1_CS1
#endif

}

/*
 ****************************
 **          MMC           **
 ****************************
 */

/*
 * MMC1 - Note: this multiplex choice cannot be used in conjunction with uart1  or uart 4.
 * Untested - example configuration for cape mmc1
 */
static void init_mmc1_pin_mux(void)
{                                                                        
    out32(conf_gpmc_ad3         , (MODE(1) | RXACTIVE));                         /* [P8  6] MMC1_DAT3  */
    out32(conf_gpmc_ad2         , (MODE(1) | RXACTIVE));                         /* [P8  5] MMC1_DAT2  */
    out32(conf_gpmc_ad1         , (MODE(1) | RXACTIVE));                         /* [P8 24] MMC1_DAT1  */
    out32(conf_gpmc_ad0         , (MODE(1) | RXACTIVE));                         /* [P8 25] MMC1_DAT0  */
    out32(conf_gpmc_csn1        , (MODE(2) | RXACTIVE | PULLUP_EN));             /* [P8 21] MMC1_CLK   */
    out32(conf_gpmc_csn2        , (MODE(2) | RXACTIVE | PULLUP_EN));             /* [P8 20] MMC1_CMD   */
    out32(conf_uart1_rxd        , (MODE(1) | RXACTIVE | PULLUP_EN));             /* [P9 26] MMC1_WP    */
    out32(conf_gpmc_wait0       , (MODE(4) | RXACTIVE));                         /* [P9 11] MMC1_CD    */
}

/*
 ****************************
 **          LCD           **
 ****************************
 */

/*
 * LCD - Note: this multiplex choice cannot be used in conjunction with hdmi.
 * Untested - example configuration for cape LCD
 */

 static void init_lcd_pin_mux(void)
{                                                            
    out32(conf_lcd_data0        , MODE(0));                                      /* [P8 45] LCD_DATA0      */
    out32(conf_lcd_data1        , MODE(0));                                      /* [P8 46] LCD_DATA1      */
    out32(conf_lcd_data2        , MODE(0));                                      /* [P8 43] LCD_DATA2      */
    out32(conf_lcd_data3        , MODE(0));                                      /* [P8 44] LCD_DATA3      */
    out32(conf_lcd_data4        , MODE(0));                                      /* [P8 41] LCD_DATA4      */
    out32(conf_lcd_data5        , MODE(0));                                      /* [P8 42] LCD_DATA5      */
    out32(conf_lcd_data6        , MODE(0));                                      /* [P8 39] LCD_DATA6      */
    out32(conf_lcd_data7        , MODE(0));                                      /* [P8 40] LCD_DATA7      */
    out32(conf_lcd_data8        , MODE(0));                                      /* [P8 37] LCD_DATA8      */
    out32(conf_lcd_data9        , MODE(0));                                      /* [P8 38] LCD_DATA9      */
    out32(conf_lcd_data10       , MODE(0));                                      /* [P8 36] LCD_DATA10     */
    out32(conf_lcd_data11       , MODE(0));                                      /* [P8 34] LCD_DATA11     */
    out32(conf_lcd_data12       , MODE(0));                                      /* [P8 35] LCD_DATA12     */
    out32(conf_lcd_data13       , MODE(0));                                      /* [P8 33] LCD_DATA13     */
    out32(conf_lcd_data14       , MODE(0));                                      /* [P8 31] LCD_DATA14     */
    out32(conf_lcd_data15       , MODE(0));                                      /* [P8 32] LCD_DATA15     */
    out32(conf_gpmc_ad15        , MODE(1));                                      /* [P8 15] LCD_DATA16     */
    out32(conf_gpmc_ad14        , MODE(1));                                      /* [P8 16] LCD_DATA17     */
    out32(conf_gpmc_ad13        , MODE(1));                                      /* [P8 11] LCD_DATA18     */
    out32(conf_gpmc_ad12        , MODE(1));                                      /* [P8 12] LCD_DATA19     */
    out32(conf_gpmc_ad11        , MODE(1));                                      /* [P8 17] LCD_DATA20     */
    out32(conf_gpmc_ad10        , MODE(1));                                      /* [P8 14] LCD_DATA21     */
    out32(conf_gpmc_ad9         , MODE(1));                                      /* [P8 13] LCD_DATA22     */
    out32(conf_gpmc_ad8         , MODE(1));                                      /* [P8 19] LCD_DATA23     */
    out32(conf_lcd_vsync        , MODE(0) | PULLUP_EN);                          /* [P8 27] LCD_VSYNC      */
    out32(conf_lcd_hsync        , MODE(0) | PULLUP_EN);                          /* [P8 29] LCD_HSYNC      */
    out32(conf_lcd_pclk         , MODE(0) | PULLUP_EN);                          /* [P8 28] LCD_PCLK       */
    out32(conf_lcd_ac_bias_en   , MODE(0) | PULLUDDIS);                          /* [P8 30] LCD_AC_BIAS_EN */
    /* backlight and/or power control */
//    out32(conf_ecap0_in_pwm0_out, MODE(7) | PULLUDDIS);                          /* [P9 42] LCD_AC_BIAS_EN */ // on LCR this line is SPI1_CS1 (EEPROM CS)
}

/*
 ****************************
 **          HDMI           **
 ****************************
 */

/*
 * HDMI - Note: this multiplex choice cannot be used in conjunction with lcd or SPI1.
 */



// NB uart0 not covered here since we don't give the option to disable it.
// rmgii1 not yet covered here.
typedef struct cape_profile
{
	int uart1;
	int uart2;
	int uart3;
	int uart4;
	int uart5;
	int i2c1;
	int i2c2;
	int spi0;
	int spi1;
	int mmc1;
	int can0;
	int can1;
	int lcd;
	int hdmi;
} CAPE_PROFILE;


CAPE_PROFILE cape_profiles[] = {
//        uart1
//        | uart2
//        | | uart3
//        | | | uart4
//        | | | | uart5
//        | | | | | i2c1
//        | | | | | | i2c2
//        | | | | | | | spi0
//        | | | | | | | | spi1
//        | | | | | | | | | mmc1 n/a
//        | | | | | | | | | | can0 n/a
//        | | | | | | | | | | | can1 n/a
//        | | | | | | | | | | | | lcd
//        | | | | | | | | | | | | | hdmi n/a
//        | | | | | | | | | | | | | | gpmc
        { 1,0,0,1,1,1,1,0,1,0,0,0,0,0 },	/* 00 - this is the default profile where no cape is present */
        { 1,0,0,1,0,1,1,0,0,1,0,0,0,1 },	/* 01 - BeagleBone Black with HDMI */ // i2c1,2 enabled, spi0,1 disabled, mmc0=SD card mmc1=eMMC
        { 1,0,0,1,0,0,1,1,1,1,0,0,0,0 },	/* 02 - BBB : i2c0,2 SPI0,1 all enabled. NB I2C1 disabled since SPI0 enabled */
        { 1,0,0,1,0,1,1,0,1,1,0,0,0,0 },	/* 03 - enable i2c1,2 SPI01 - for testing*/
        { 0,0,0,0,0,1,1,1,1,0,0,0,1,0 },	/* 04 - LCR : all i2c,SPI0 enabled. TBC gpmc */
        { 0,0,0,0,0,0,0,0,0,0,0,0,0,0 },	/* 05 - add your own profile here */
};

void init_pinmux_capes(int profile, int showchoices)
{
	// Any UARTS?
	if (cape_profiles[profile].uart1)
	{
		if (showchoices) kprintf("uart1\n");
		init_uart1_pin_mux();
	}
	if (cape_profiles[profile].uart2)
	{
		if (showchoices) kprintf("uart2\n");
		init_uart2_pin_mux();
	}
		
	// Any i2c?
	if (cape_profiles[profile].i2c1)
	{
		if (showchoices) kprintf("i2c1\n");
		init_i2c1_pin_mux();
	}
	if (cape_profiles[profile].i2c2)
	{
		if (showchoices) kprintf("i2c2\n");
		init_i2c2_pin_mux();
	}
	
	if (cape_profiles[profile].spi0)
	{
		if (showchoices) kprintf("spi0\n");
		init_spi0_pin_mux();
	}

	if (cape_profiles[profile].spi1)
	{
		if (showchoices) kprintf("spi1\n");
		init_spi1_pin_mux();
	}


	

	// LCD?
	if (cape_profiles[profile].lcd)
	{
		if (showchoices) kprintf("lcd\n");
		init_lcd_pin_mux();
	}


}

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/hardware/startup/boards/ti-am335x/beaglebone/init_pinmux.c $ $Rev: 761911 $")
#endif
